Class: Fluent::Compat::Input

Inherits:
Plugin::Input
  • Object
show all
Defined in:
lib/fluent/compat/input.rb

Instance Method Summary collapse

Constructor Details

#initializeInput

TODO: warn when deprecated



27
28
29
30
31
32
# File 'lib/fluent/compat/input.rb', line 27

def initialize
  super
  unless self.class.ancestors.include?(Fluent::Compat::CallSuperMixin)
    self.class.prepend Fluent::Compat::CallSuperMixin
  end
end

Instance Method Details

#before_shutdownObject



40
41
42
# File 'lib/fluent/compat/input.rb', line 40

def before_shutdown
  super
end

#shutdownObject



44
45
46
# File 'lib/fluent/compat/input.rb', line 44

def shutdown
  super
end

#startObject

These definitions are to get instance methods of superclass of 3rd party plugins to make it sure to call super



36
37
38
# File 'lib/fluent/compat/input.rb', line 36

def start
  super
end