Class: Fluent::Plugin::Buffer::FileChunk
- Defined in:
- lib/fluent/plugin/buffer/file_chunk.rb
Defined Under Namespace
Classes: FileChunkError
Constant Summary collapse
- BUFFER_HEADER =
"\xc1\x00".force_encoding(Encoding::ASCII_8BIT).freeze
Instance Attribute Summary collapse
-
#meta_path ⇒ Object
readonly
state: b/q - ‘b’(on stage, compatible with v0.12), ‘q’(enqueued) path_prefix: path prefix string, ended with ‘.’ path_suffix: path suffix string, like ‘.log’ (or any other user specified).
-
#path ⇒ Object
readonly
state: b/q - ‘b’(on stage, compatible with v0.12), ‘q’(enqueued) path_prefix: path prefix string, ended with ‘.’ path_suffix: path suffix string, like ‘.log’ (or any other user specified).
-
#permission ⇒ Object
readonly
state: b/q - ‘b’(on stage, compatible with v0.12), ‘q’(enqueued) path_prefix: path prefix string, ended with ‘.’ path_suffix: path suffix string, like ‘.log’ (or any other user specified).
Attributes inherited from Chunk
Class Method Summary collapse
- .assume_chunk_state(path) ⇒ Object
- .generate_queued_chunk_path(path, unique_id) ⇒ Object
- .generate_stage_chunk_path(path, unique_id) ⇒ Object
-
.unique_id_from_path(path) ⇒ Object
used only for queued v0.12 buffer path or broken files.
Instance Method Summary collapse
- #bytesize ⇒ Object
- #close ⇒ Object
- #commit ⇒ Object
- #concat(bulk, bulk_size) ⇒ Object
- #create_new_chunk(path, perm) ⇒ Object
- #empty? ⇒ Boolean
- #enqueued! ⇒ Object
- #file_rename(file, old_path, new_path, callback = nil) ⇒ Object
-
#initialize(metadata, path, mode, perm: nil, compress: :text, decompression_size_limit: Compressable::DEFAULT_DECOMPRESSION_SIZE_LIMIT) ⇒ FileChunk
constructor
A new instance of FileChunk.
- #load_existing_enqueued_chunk(path) ⇒ Object
- #load_existing_staged_chunk(path) ⇒ Object
- #open(**kwargs, &block) ⇒ Object
- #purge ⇒ Object
- #read(**kwargs) ⇒ Object
- #restore_metadata(bindata) ⇒ Object
- #restore_metadata_partially(chunk) ⇒ Object
- #rollback ⇒ Object
- #size ⇒ Object
- #write_metadata(update: true) ⇒ Object
Methods inherited from Chunk
#append, #closed?, #created_at, #modified_at, #queued?, #raw_create_at, #raw_modified_at, #staged!, #staged?, #unstaged!, #unstaged?, #writable?, #write_to
Methods included from UniqueId::Mixin
#dump_unique_id_hex, #generate_unique_id
Constructor Details
#initialize(metadata, path, mode, perm: nil, compress: :text, decompression_size_limit: Compressable::DEFAULT_DECOMPRESSION_SIZE_LIMIT) ⇒ FileChunk
Returns a new instance of FileChunk.
42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 42 def initialize(, path, mode, perm: nil, compress: :text, decompression_size_limit: Compressable::DEFAULT_DECOMPRESSION_SIZE_LIMIT) super(, compress: compress, decompression_size_limit: decompression_size_limit) perm ||= Fluent::DEFAULT_FILE_PERMISSION @permission = perm.is_a?(String) ? perm.to_i(8) : perm @bytesize = @size = @adding_bytes = @adding_size = 0 @meta = nil case mode when :create then create_new_chunk(path, @permission) when :staged then load_existing_staged_chunk(path) when :queued then load_existing_enqueued_chunk(path) else raise ArgumentError, "Invalid file chunk mode: #{mode}" end end |
Instance Attribute Details
#meta_path ⇒ Object (readonly)
state: b/q - ‘b’(on stage, compatible with v0.12), ‘q’(enqueued) path_prefix: path prefix string, ended with ‘.’ path_suffix: path suffix string, like ‘.log’ (or any other user specified)
40 41 42 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 40 def @meta_path end |
#path ⇒ Object (readonly)
state: b/q - ‘b’(on stage, compatible with v0.12), ‘q’(enqueued) path_prefix: path prefix string, ended with ‘.’ path_suffix: path suffix string, like ‘.log’ (or any other user specified)
40 41 42 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 40 def path @path end |
#permission ⇒ Object (readonly)
state: b/q - ‘b’(on stage, compatible with v0.12), ‘q’(enqueued) path_prefix: path prefix string, ended with ‘.’ path_suffix: path suffix string, like ‘.log’ (or any other user specified)
40 41 42 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 40 def @permission end |
Class Method Details
.assume_chunk_state(path) ⇒ Object
176 177 178 179 180 181 182 183 184 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 176 def self.assume_chunk_state(path) if /\.(b|q)([0-9a-f]+)\.[^\/]*\Z/n =~ path # //n switch means explicit 'ASCII-8BIT' pattern $1 == 'b' ? :staged : :queued else # files which matches to glob of buffer file pattern # it includes files which are created by out_file :unknown end end |
.generate_queued_chunk_path(path, unique_id) ⇒ Object
198 199 200 201 202 203 204 205 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 198 def self.generate_queued_chunk_path(path, unique_id) chunk_id = Fluent::UniqueId.hex(unique_id) if path.index(".b#{chunk_id}.") path.sub(".b#{chunk_id}.", ".q#{chunk_id}.") else # for unexpected cases (ex: users rename files while opened by fluentd) path + ".q#{chunk_id}.chunk" end end |
.generate_stage_chunk_path(path, unique_id) ⇒ Object
186 187 188 189 190 191 192 193 194 195 196 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 186 def self.generate_stage_chunk_path(path, unique_id) pos = path.index('.*.') raise "BUG: buffer chunk path on stage MUST have '.*.'" unless pos prefix = path[0...pos] suffix = path[(pos+3)..-1] chunk_id = Fluent::UniqueId.hex(unique_id) state = 'b' "#{prefix}.#{state}#{chunk_id}.#{suffix}" end |
.unique_id_from_path(path) ⇒ Object
used only for queued v0.12 buffer path or broken files
208 209 210 211 212 213 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 208 def self.unique_id_from_path(path) if /\.(b|q)([0-9a-f]+)\.[^\/]*\Z/n =~ path # //n switch means explicit 'ASCII-8BIT' pattern return $2.scan(/../).map{|x| x.to_i(16) }.pack('C*') end nil end |
Instance Method Details
#bytesize ⇒ Object
90 91 92 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 90 def bytesize @bytesize + @adding_bytes end |
#close ⇒ Object
146 147 148 149 150 151 152 153 154 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 146 def close super size = @chunk.size @chunk.close @meta.close if @meta # meta may be missing if chunk is queued at first if size == 0 File.unlink(@path, @meta_path) end end |
#commit ⇒ Object
68 69 70 71 72 73 74 75 76 77 78 79 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 68 def commit # this should be at first: of course, this operation may fail @commit_position = @chunk.pos @size += @adding_size @bytesize += @adding_bytes @adding_bytes = @adding_size = 0 @modified_at = Fluent::Clock.real_now @modified_at_object = nil true end |
#concat(bulk, bulk_size) ⇒ Object
58 59 60 61 62 63 64 65 66 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 58 def concat(bulk, bulk_size) raise "BUG: concatenating to unwritable chunk, now '#{self.state}'" unless self.writable? bulk.force_encoding(Encoding::ASCII_8BIT) @chunk.write bulk @adding_bytes += bulk.bytesize @adding_size += bulk_size true end |
#create_new_chunk(path, perm) ⇒ Object
282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 282 def create_new_chunk(path, perm) @path = self.class.generate_stage_chunk_path(path, @unique_id) @meta_path = @path + '.meta' begin @chunk = File.open(@path, 'wb+', perm) @chunk.set_encoding(Encoding::ASCII_8BIT) @chunk.sync = true @chunk.binmode rescue => e # Here assumes "Too many open files" like recoverable error so raising BufferOverflowError. # If other cases are possible, we will change error handling with proper classes. raise BufferOverflowError, "can't create buffer file for #{path}. Stop creating buffer files: error = #{e}" end begin @meta = File.open(@meta_path, 'wb', perm) @meta.set_encoding(Encoding::ASCII_8BIT) @meta.sync = true @meta.binmode (update: false) rescue => e # This case is easier than enqueued!. Just removing pre-create buffer file @chunk.close rescue nil File.unlink(@path) rescue nil if @meta # ensure to unlink when #write_metadata fails @meta.close rescue nil File.unlink(@meta_path) rescue nil end # Same as @chunk case. See above raise BufferOverflowError, "can't create buffer metadata for #{path}. Stop creating buffer files: error = #{e}" end @state = :unstaged @bytesize = 0 @commit_position = @chunk.pos # must be 0 @adding_bytes = 0 @adding_size = 0 end |
#empty? ⇒ Boolean
98 99 100 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 98 def empty? @bytesize == 0 end |
#enqueued! ⇒ Object
102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 102 def enqueued! return unless self.staged? new_chunk_path = self.class.generate_queued_chunk_path(@path, @unique_id) = new_chunk_path + '.meta' (update: false) # re-write metadata w/ finalized records begin file_rename(@chunk, @path, new_chunk_path, ->(new_io) { @chunk = new_io }) rescue => e begin file_rename(@chunk, new_chunk_path, @path, ->(new_io) { @chunk = new_io }) if File.exist?(new_chunk_path) rescue => re # In this point, restore buffer state is hard because previous `file_rename` failed by resource problem. # Retry is one possible approach but it may cause livelock under limited resources or high load environment. # So we ignore such errors for now and log better message instead. # "Too many open files" should be fixed by proper buffer configuration and system setting. raise "can't enqueue buffer file and failed to restore. This may causes inconsistent state: path = #{@path}, error = '#{e}', retry error = '#{re}'" else raise "can't enqueue buffer file: path = #{@path}, error = '#{e}'" end end begin file_rename(@meta, @meta_path, , ->(new_io) { @meta = new_io }) rescue => e begin file_rename(@chunk, new_chunk_path, @path, ->(new_io) { @chunk = new_io }) if File.exist?(new_chunk_path) file_rename(@meta, , @meta_path, ->(new_io) { @meta = new_io }) if File.exist?() rescue => re # See above raise "can't enqueue buffer metadata and failed to restore. This may causes inconsistent state: path = #{@meta_path}, error = '#{e}', retry error = '#{re}'" else raise "can't enqueue buffer metadata: path = #{@meta_path}, error = '#{e}'" end end @path = new_chunk_path @meta_path = super end |
#file_rename(file, old_path, new_path, callback = nil) ⇒ Object
265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 265 def file_rename(file, old_path, new_path, callback=nil) pos = file.pos if Fluent.windows? file.close File.rename(old_path, new_path) file = File.open(new_path, 'rb', @permission) else File.rename(old_path, new_path) file.reopen(new_path, 'rb') end file.set_encoding(Encoding::ASCII_8BIT) file.sync = true file.binmode file.pos = pos callback.call(file) if callback end |
#load_existing_enqueued_chunk(path) ⇒ Object
373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 373 def load_existing_enqueued_chunk(path) @path = path raise FileChunkError, "enqueued file chunk is empty" if File.size(@path).zero? @chunk = File.open(@path, 'rb') @chunk.set_encoding(Encoding::ASCII_8BIT) @chunk.binmode @chunk.seek(0, IO::SEEK_SET) @bytesize = @chunk.size @commit_position = @chunk.size @meta_path = @path + '.meta' if File.readable?(@meta_path) begin (File.open(@meta_path){|f| f.set_encoding(Encoding::ASCII_8BIT); f.binmode; f.read }) rescue => e @chunk.close raise FileChunkError, "enqueued meta file is broken. #{e.}" end else (@chunk) end @state = :queued end |
#load_existing_staged_chunk(path) ⇒ Object
323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 323 def load_existing_staged_chunk(path) @path = path @meta_path = @path + '.meta' @meta = nil # staging buffer chunk without metadata is classic buffer chunk file # and it should be enqueued immediately if File.exist?(@meta_path) raise FileChunkError, "staged file chunk is empty" if File.size(@path).zero? @chunk = File.open(@path, 'rb+') @chunk.set_encoding(Encoding::ASCII_8BIT) @chunk.sync = true @chunk.seek(0, IO::SEEK_END) @chunk.binmode @meta = File.open(@meta_path, 'rb+') @meta.set_encoding(Encoding::ASCII_8BIT) @meta.sync = true @meta.binmode begin (@meta.read) rescue => e @chunk.close @meta.close raise FileChunkError, "staged meta file is broken. #{e.}" end @meta.seek(0, IO::SEEK_SET) @state = :staged @bytesize = @chunk.size @commit_position = @chunk.pos @adding_bytes = 0 @adding_size = 0 else # classic buffer chunk - read only chunk @chunk = File.open(@path, 'rb') @chunk.set_encoding(Encoding::ASCII_8BIT) @chunk.binmode @chunk.seek(0, IO::SEEK_SET) @state = :queued @bytesize = @chunk.size (@chunk) @commit_position = @chunk.size @unique_id = self.class.unique_id_from_path(@path) || @unique_id end end |
#open(**kwargs, &block) ⇒ Object
169 170 171 172 173 174 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 169 def open(**kwargs, &block) @chunk.seek(0, IO::SEEK_SET) val = yield @chunk @chunk.seek(0, IO::SEEK_END) if self.staged? val end |
#purge ⇒ Object
156 157 158 159 160 161 162 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 156 def purge super @chunk.close @meta.close if @meta @bytesize = @size = @adding_bytes = @adding_size = 0 File.unlink(@path, @meta_path) end |
#read(**kwargs) ⇒ Object
164 165 166 167 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 164 def read(**kwargs) @chunk.seek(0, IO::SEEK_SET) @chunk.read end |
#restore_metadata(bindata) ⇒ Object
215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 215 def (bindata) data = (bindata) unless data # old type of restore data = Fluent::MessagePackFactory.msgpack_unpacker(symbolize_keys: true).feed(bindata).read rescue {} end raise FileChunkError, "invalid meta data" if data.nil? || !data.is_a?(Hash) raise FileChunkError, "invalid unique_id" unless data[:id] raise FileChunkError, "invalid created_at" unless data[:c].to_i > 0 raise FileChunkError, "invalid modified_at" unless data[:m].to_i > 0 now = Fluent::Clock.real_now @unique_id = data[:id] @size = data[:s] || 0 @created_at = data[:c] @modified_at = data[:m] @metadata.timekey = data[:timekey] @metadata.tag = data[:tag] @metadata.variables = data[:variables] @metadata.seq = data[:seq] || 0 end |
#restore_metadata_partially(chunk) ⇒ Object
240 241 242 243 244 245 246 247 248 249 250 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 240 def (chunk) @unique_id = self.class.unique_id_from_path(chunk.path) || @unique_id @size = 0 @created_at = chunk.ctime.to_i # birthtime isn't supported on Windows (and Travis?) @modified_at = chunk.mtime.to_i @metadata.timekey = nil @metadata.tag = nil @metadata.variables = nil @metadata.seq = 0 end |
#rollback ⇒ Object
81 82 83 84 85 86 87 88 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 81 def rollback if @chunk.pos != @commit_position @chunk.seek(@commit_position, IO::SEEK_SET) @chunk.truncate(@commit_position) end @adding_bytes = @adding_size = 0 true end |
#size ⇒ Object
94 95 96 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 94 def size @size + @adding_size end |
#write_metadata(update: true) ⇒ Object
252 253 254 255 256 257 258 259 260 261 262 263 |
# File 'lib/fluent/plugin/buffer/file_chunk.rb', line 252 def (update: true) data = @metadata.to_h.merge({ id: @unique_id, s: (update ? @size + @adding_size : @size), c: @created_at, m: (update ? Fluent::Clock.real_now : @modified_at), }) bin = Fluent::MessagePackFactory.thread_local_msgpack_packer.pack(data).full_pack size = [bin.bytesize].pack('N') @meta.seek(0, IO::SEEK_SET) @meta.write(BUFFER_HEADER + size + bin) end |